import QUB
from Buttons import *
from Dialogs import *
from qubtree import Node
import math

def GenEvens(gen):
    while 1:
        gen.next()
        yield gen.next()

def GenOdds(gen):
    while 1:
        yield gen.next()
        gen.next()

def children(tree, name=None):
    if name == None:
        child = tree.child
    else:
        child = tree.find(name)
    while not child.isNull:
        yield child
        if name == None:
            child = child.sibling
        else:
            child = child.nextSameName()

def GenDataSegments(dataset):
    for seg in children(dataset, 'Segment'):
        QUB.fillData(seg)
        yield seg
        seg.remove(seg['Channel'])

def GenSelsBySlope(segments, direction):
    if direction < 0:
        isDir = lambda x, y: x > y
    else:
        isDir = lambda x, y: x < y
    for seg in segments:
        first = seg.data[0]
        data = seg['Channel'].data
        last = None
        for i, x in enumerate(data):
            if last == None:
                spot = 0
            elif not isDir(last, x):
                if spot != (i-1):
                    yield (first+spot, first+i-1)
                spot = i
            last = x
        if spot != (len(data) - 1):
            yield (first+spot, first + len(data) - 1)

def GenSelsThresh(sels, ptsThresh):
    for f, l in sels:
        if (l - f + 1) >= ptsThresh:
            yield (f, l)
                
def CSDrops(props):
    direction = {'increasing':1, 'decreasing':-1}[ props["PiezoDir"] ]
    MkSkip = {'all'   : lambda x: x,
              'odds'  : GenOdds,
              'evens' : GenEvens }[ props['keep'] ]
    
    ### Get The Data ###
    dataFile = QUB.DataView.currData
    data=QUB.getChosenData()
    dataset = data.find('DataSet')
    dataset['ActiveChannel'].data = props['ChnlPiezo']
    sels = [sel for sel in GenSelsThresh( MkSkip( GenSelsBySlope( GenDataSegments(dataset), direction ) ), props['ptsThresh'])]
    if len(sels) and props['Trim'] != 'neither':
        N = min([l-f+1 for f,l in sels])
        if props['Trim'] == 'left':
            Trim = lambda f, l: (f + ((l-f+1)-N), l)
        else:
            Trim = lambda f, l: (f, l - ((l-f+1)-N))
        sels = [Trim(f, l) for f,l in sels]
    dataFile.setList(props['ListName'], sels)
    #list = dataFile.lists.byName(props['ListName'])
    #list.clear()
    #for f, l in sels:
    #    list.add(f, l)

AddTaskButton( 'CS Drops', # button text
               CSDrops,   # function
               'CSDrop',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "List where piezo moving down",   # tool-tip
               "List where piezo moving down",   # description in "Edit Buttons"
               "CS Drops settings:", # caption for the settings dialog
               [DialogText("ListName", "List name:", "Drops", acceptString),
                DialogText("ChnlPiezo", "Piezo Channel:", 1, acceptIntGreaterThan(-1)),
                DialogRadioGroup("PiezoDir", "List where piezo is", ['increasing', 'decreasing'], 'increasing'),
                DialogRadioGroup("keep", 'Keep selections:', ['all', 'odds', 'evens'], 'all'),
                DialogText("ptsThresh", 'Ignore sels shorter than (points):',400, acceptIntGreaterThan(0)),
                DialogRadioGroup("Trim", 'For uniform width, trim', ['left', 'right', 'neither'], 'left')],
               'Preprocessing' )

